@@ -30,7 +30,7 @@ use constant DEBUG => 0;
 use vars qw(@ISA $AUTOLOAD %CONSTANT_TAGS $VERSION);
 @ISA = qw(Bio::DB::Das::Chado::Segment Bio::SeqFeatureI
           Bio::Root::Root);
-$VERSION = 0.30;
+$VERSION = 0.31;
 
 %CONSTANT_TAGS = ();
 
@@ -468,49 +468,31 @@ sub get_tagset_values {
 
 sub gff_string {
   my $self = shift;
-  my ($recurse,$parent) = @_;
-  my ($start,$stop) = ($self->start,$self->stop);
+  my $recurse = shift;
+  my $feature_id=$self->feature_id; 
 
-  # the defined() tests prevent uninitialized variable warnings, when dealing with clone objects
-  # whose endpoints may be undefined
-  ($start,$stop) = ($stop,$start) if defined($start) && defined($stop) && $start > $stop;
+  my $gff_init_query = "SELECT ref,source,type,fstart,fend,score,strand,phase FROM gff3view WHERE feature_id=$feature_id"; 
+  my @row_ary = $self->factory->dbh->selectrow_array($gff_init_query);
+  my $string = join("\t",@row_ary)."\t";
 
-  my $strand = ('-','.','+')[$self->strand+1];
-  my $ref = $self->refseq;
-  my $n   = ref($ref) ? $ref->name : $ref;
-  my $phase = $self->phase;
-  $phase = '.' unless defined $phase;
+  my $gff_atts_query = "SELECT type,attribute from gff3atts where feature_id=?";
+  my $sth = $self->factory->dbh->prepare($gff_atts_query);
+  $sth->execute($feature_id);
 
-  my ($class,$name) = ('','');
-  my @group;
-  if (my $g = $self->group) {
-    $class = $g->can('class') && $g->class ? $g->class : '';
-    $name  = $g->can('name')  && $g->name  ? $g->name  : '';
-    $name  = "$class:$name" if length($class) and length($name);
-    push @group,[ID =>  $name] if !defined($parent) || $name ne $parent;
-  }
+  while (my $hashref = $sth->fetchrow_hashref()) {
+      my $attribute = uri_escape($$hashref{'attribute'});
+      next unless $attribute;
+      $string .= "$$hashref{'type'}=$attribute;";
+  } 
 
-  push @group,[Parent => $parent] if defined $parent && $parent ne '';
-
-  if (my $t = $self->target) {
-    $strand = '-' if $t->stop < $t->start;
-    push @group, $self->flatten_target($t,3);
-  }
-
-  my @attributes = $self->attributes;
-  while (@attributes) {
-    push @group,[shift(@attributes),shift(@attributes)]
-  }
-  my $group_field = join ';',map {join '=',uri_escape($_->[0]),uri_escape($_->[1])} grep {$_->[0] =~ /\S/ and $_->[1] =~ /\S/} @group;
-  my $string = join("\t",$n,$self->source||'.',$self->method||'.',$start||'.',$stop||'.',
-                    $self->score||'.',$strand||'.',$phase||'.',$group_field);
-  $string .= "\n";
   if ($recurse) {
-    foreach ($self->sub_SeqFeature) {
-      $string .= $_->gff_string(1,$name);
-    }
+      foreach($self->sub_SeqFeature) {
+          $string .= "\n";
+          $string .= $_->gff_string(1);
+      }
   }
-  $string;
+
+  return $string;
 }
 
 =head2 has_tag()
@@ -560,7 +542,8 @@ sub has_tag {
 
   Title   : seq_id
   Usage   : $obj->seq_id($newval)
-  Function: ???
+  Function: Set or get the name of the reference sequence that the feature 
+            resides on.
   Returns : value of seq_id (a scalar)
   Args    : on set, new value (a scalar or undef, optional)
 
@@ -571,7 +554,23 @@ sub seq_id {
   my $self = shift;
 
   return $self->{'seq_id'} = shift if @_;
-  return $self->{'seq_id'};
+  return $self->{'seq_id'} if defined $self->{'seq_id'};
+
+  #OK, no seq_id found, we'll try to find one.
+
+  my $feature_id = $self->feature_id;
+  return unless $feature_id;
+
+  my $query =<<END
+  SELECT COALESCE(f.name,f.uniquename) AS seq_id
+  FROM feature f join featureloc fl ON (f.feature_id = fl.srcfeature_id) 
+  WHERE fl.feature_id = $feature_id AND fl.rank = 0
+END
+;
+
+  my ($seq_id) = $self->factory->dbh->selectrow_array($query);
+
+  return $self->{'seq_id'} = $seq_id;
 }
 
 ######################################################################
@@ -825,12 +824,20 @@ sub target {
 
 sub all_tags {
   my $self = shift;
-  my @tags = keys %CONSTANT_TAGS;
-  # autogenerated methods
-  if (my $subfeat = $self->subfeatures) {
-    push @tags,keys %$subfeat;
+  my $feature_id = $self->feature_id;
+
+  my $tag_query = $self->factory->dbh->prepare(
+    "SELECT distinct type FROM gffatts WHERE feature_id=?"
+  );
+
+  $tag_query->execute($feature_id);
+
+  my @tags;
+  while (my @tag = $tag_query->fetchrow_array) {
+    push @tags, $tag[0];
   }
-  @tags;
+
+  return @tags;
 }
 
 =head2 source()
